@@ -18,7 +18,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 22 Apr 2012                                              }
+{ last modified: 19 May 2012                                              }
 {*************************************************************************/
 
 #include "qkineticwidget.h"
@@ -316,6 +316,15 @@ void QKineticWidget::deleteAllChildren()
 
 void QKineticWidget::repositionChildrenV(int itemsHeight, int rows)
 {
+    float _x = getX();
+    float _y = getY();
+    float _spx = getSpeedX();
+    float _spy = getSpeedY();
+    setSpeedX(0);
+    setSpeedY(0);
+    setX(0);
+    setY(0);
+
     int ww = width()/rows;
     int c = 0;
     int rowHeight = 0;
@@ -355,6 +364,11 @@ void QKineticWidget::repositionChildrenV(int itemsHeight, int rows)
     }
 
     setInnerHeight(y + rowHeight);
+
+    setX(_x);
+    setY(_y);
+    setSpeedX(_spx);
+    setSpeedY(_spy);
 }
 
 void QKineticWidget::centerWidgetV(const QWidget *widget)
