@@ -18,7 +18,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 31 Mar 2012                                              }
+{ last modified: 19 May 2012                                              }
 {*************************************************************************/
 
 #include "fsman.h"
@@ -32,6 +32,9 @@ FSMan::FSMan(QObject *parent) :
     doUpOnFail = true;
     doUseCallbackFilters = true;
     curDir = "";
+    searching = false;
+    searchRx.setPatternSyntax(QRegExp::Wildcard);
+    searchRx.setCaseSensitivity(Qt::CaseInsensitive);
 }
 
 bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
@@ -50,14 +53,14 @@ QString FSMan::which(const QString& program)
 #endif
 }
 
-void FSMan::_fetchForce()
+void FSMan::_fetchForce(const QString &_dir, QStringList &_dirs, QStringList &_files)
 {
-    FSMan::ls(curDir, dirs, files, doUseNativeMethods);
+    FSMan::ls(_dir, _dirs, _files, doUseNativeMethods);
     QRegExp rx;
     rx.setPatternSyntax(QRegExp::WildcardUnix);
     rx.setCaseSensitivity(Qt::CaseInsensitive);
     QStringList filesFiltered;
-    foreach(QString name, files)
+    foreach(QString name, _files)
     {
         foreach(QString filter, nameFilters)
         {
@@ -69,56 +72,61 @@ void FSMan::_fetchForce()
             }
         }
     }
-    files = filesFiltered;
+    _files = filesFiltered;
 
     if(doUseCallbackFilters)
     {
         QStringList dirsFiltered;
         filesFiltered.clear();
         bool passed;
-        foreach(QString dirName, dirs)
+        foreach(QString dirName, _dirs)
         {
             passed = true;
-            emit onDirFilter(&passed, dirName, curDir);
+            emit onDirFilter(&passed, dirName, _dir);
             if(passed)
                 dirsFiltered.append(dirName);
         }
-        foreach(QString fileName, files)
+        foreach(QString fileName, _files)
         {
             passed = true;
-            emit onFileFilter(&passed, fileName, curDir);
+            emit onFileFilter(&passed, fileName, _dir);
             if(passed)
                 filesFiltered.append(fileName);
         }
-        files = filesFiltered;
-        dirs = dirsFiltered;
+        _files = filesFiltered;
+        _dirs = dirsFiltered;
     }
 }
 
-void FSMan::_fetch()
+void FSMan::nextSearchStep()
+{
+    QTimer::singleShot(0, this, SLOT(onSearchStep()));
+}
+
+void FSMan::_fetch(const QString& _dir, QStringList& _dirs, QStringList& _files)
 {
     if(doFetchLists)
     {
         if(doUseCache)
         {
-            if(cache.contains(curDir))
+            if(cache.contains(_dir))
             {
-                FSManCacheRecord crec = cache.value(curDir);
-                files = crec.files;
-                dirs = crec.dirs;
+                FSManCacheRecord crec = cache.value(_dir);
+                _files = crec.files;
+                _dirs = crec.dirs;
             }
             else
             {
-                _fetchForce();
+                _fetchForce(_dir, _dirs, _files);
                 FSManCacheRecord rec;
-                rec.files = files;
-                rec.dirs = dirs;
-                cache.insert(curDir, rec);
+                rec.files = _files;
+                rec.dirs = _dirs;
+                cache.insert(_dir, rec);
             }
         }
         else
         {
-            _fetchForce();
+            _fetchForce(_dir, _dirs, _files);
         }
     }
 }
@@ -134,7 +142,7 @@ bool FSMan::_cd()
         if(lastDir == curDir)
             break;
     }
-    _fetch();
+    _fetch(curDir, dirs, files);
     return result;
 }
 
@@ -169,7 +177,7 @@ bool FSMan::cd(const QString& dirPath)
     {
         if(cache.contains(curDir))
         {
-            _fetch();
+            _fetch(curDir, dirs, files);
         }
         else
         {
@@ -294,6 +302,42 @@ QString FSMan::shellEscape(const QString& line)
     return result;
 }
 
+void FSMan::startSearch(const QString &expression, const QString startDir)
+{
+    stopSearch();
+    if(expression.isEmpty())
+    {
+        emit onSearchFinished();
+        return;
+    }
+
+    QString _dir = startDir;
+
+    if(_dir.isEmpty())
+        _dir = curDir;
+    if(_dir.isEmpty())
+    {
+#ifdef Q_OS_WIN32
+        _dir = "C:/";
+#else
+        _dir = "/";
+#endif
+    }
+    else
+    {
+        _dir.replace("\\", "/");
+        if(_dir.endsWith("/") && (_dir.size() > 1))
+            _dir.chop(1);
+    }
+
+    searchDirs.clear();
+    searchDirs.append(_dir);
+    searching = true;
+    searchRx.setPattern(expression);
+
+    nextSearchStep();
+}
+
 void FSMan::ls(const QString& path, QStringList& dirsList, QStringList& filesList, bool useNativeMethod)
 {
     dirsList.clear();
@@ -373,9 +417,45 @@ QString FSMan::getProcessOutput(const QString& cmd, const QStringList& args)
 
 QString FSMan::getProcessOutput(const QString& cmd)
 {
-     return getProcessOutput(cmd, QStringList());
+    return getProcessOutput(cmd, QStringList());
 }
 
+void FSMan::onSearchStep()
+{
+    if(!searching)
+        return;
 
+    QString _dir = searchDirs.takeLast();
 
+    QStringList _dirs;
+    QStringList _files;
 
+    _fetch(_dir, _dirs, _files);
+
+    QString _appendDir = _dir;
+    if(_appendDir.endsWith("/"))
+        _appendDir.chop(1);
+
+    foreach(QString item, _dirs)
+    {
+        if(searchRx.indexIn(item) >= 0)
+            emit onSearchResult(true, item, _dir);
+        searchDirs.append(_appendDir + "/" + item);
+    }
+
+    foreach(QString item, _files)
+    {
+        if(searchRx.indexIn(item) >= 0)
+            emit onSearchResult(false, item, _dir);
+    }
+
+    if(searchDirs.isEmpty())
+    {
+        stopSearch();
+        emit onSearchFinished();
+    }
+    else
+    {
+        nextSearchStep();
+    }
+}
