@@ -21,7 +21,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 25 Mar 2012                                              }
+{ last modified: 26 May 2012                                              }
 {*************************************************************************/
 
 #include "equalizerwindow.h"
@@ -30,7 +30,6 @@
 EqualizerWindow::EqualizerWindow(QWidget *parent) :
     CommonDialog(parent)
 {
-    eq = PLAYER->getEqualizer();
     presetOff = QString("<")+tr("off", "presetLabel")+QString(">");
     isChanged = false;
     initGui();
@@ -72,7 +71,7 @@ void EqualizerWindow::initGui()
     connect(btnSave, SIGNAL(clicked()), SLOT(onSaveClick()));
     connect(btnDelete, SIGNAL(clicked()), SLOT(onDeleteClick()));
 
-    bars = (QWidget**)(malloc(sizeof(QWidget*)*eq->getBandsCount()));
+    bars = (QWidget**)(malloc(sizeof(QWidget*)*eq()->getBandsCount()));
     QWidget* bar;
     QSlider* slider;
     QLabel* labelFreq;
@@ -80,15 +79,15 @@ void EqualizerWindow::initGui()
     QString s;
     int f;
 
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
     {
         bar = new QWidget(this);
 
         slider = new QSlider(bar);
-        slider->setMinimum(eq->getBandLowerBound(a));
-        slider->setMaximum(eq->getBandUpperBound(a));
+        slider->setMinimum(eq()->getBandLowerBound(a));
+        slider->setMaximum(eq()->getBandUpperBound(a));
 
-        f = eq->getBandFrequency(a);
+        f = eq()->getBandFrequency(a);
         if(f<1000)
         {
             s.setNum(f);
@@ -125,7 +124,7 @@ void EqualizerWindow::initGui()
 
 void EqualizerWindow::deinitGui()
 {
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
     {
         foreach(QObject* child, bars[a]->children())
             delete child;
@@ -153,39 +152,19 @@ void EqualizerWindow::reposition(const QSize &sz)
     QLabel* labelLevel;
 
     int w = sz.width();
-    int barWidth = w / eq->getBandsCount();
+    int barWidth = w / eq()->getBandsCount();
     int h = sz.height();
     int sp = CONTROLBUTTON_SIZE + CONTROLBUTTON_PADDING;
 
     bool showFreq = true;
 
-/*
-    for(int a=0; a<eq->getBandsCount(); a++)
-    {
-        bar = bars[a];
-        labelFreq = (QLabel*)(bar->children().at(1));
-        if(labelFreq->fontMetrics().width(labelFreq->text()) < barWidth)
-        {
-            showFreq = false;
-            break;
-        }
-    }
-
-    for(int a=0; a<eq->getBandsCount(); a++)
-    {
-        bar = bars[a];
-        labelFreq = (QLabel*)(bar->children().at(1));
-        labelFreq->setVisible(showFreq);
-    }
-*/
-
     int labelFreqHeight;
     if(showFreq)
         labelFreqHeight = ((QLabel*)(bars[0]->children().at(1)))->fontMetrics().height();
     else
         labelFreqHeight = 0;
 
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
     {
         bar = bars[a];
         slider = (QSlider*)(bar->children().at(0));
@@ -220,7 +199,7 @@ void EqualizerWindow::reposition(const QSize &sz)
 
 void EqualizerWindow::updateLevels()
 {
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
         updateLevel(a);
 }
 
@@ -228,7 +207,7 @@ void EqualizerWindow::updateLevel(int bandIndex)
 {
     QLabel* label = (QLabel*)(bars[bandIndex]->children().at(2));
     QSlider* slider = (QSlider*)(bars[bandIndex]->children().at(0));
-    int level = qRound(eq->getBandLevel(bandIndex));
+    int level = qRound(eq()->getBandLevel(bandIndex));
     slider->setValue(level);
     updateLevelLabel(label, level);
 }
@@ -245,7 +224,7 @@ void EqualizerWindow::updateLevelLabel(QLabel *label, int level)
 
 int EqualizerWindow::getBarIndex(QWidget *bar)
 {
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
         if(bars[a] == bar)
             return a;
     return -1;
@@ -270,13 +249,13 @@ void EqualizerWindow::selectPreset(const QString &presetName)
 {
     if(presetName.isEmpty())
     {
-        eq->setDefaultLevels();
+        eq()->setDefaultLevels();
         PLAYER->setEqualizerEnabled(false);
         setControlsEnabled(false);
     }
     else
     {
-        eq->loadFromPreset(presetName);
+        eq()->loadFromPreset(presetName);
         PLAYER->setEqualizerEnabled(true);
         setControlsEnabled(true);
     }
@@ -289,7 +268,7 @@ void EqualizerWindow::selectPreset(const QString &presetName)
 
 void EqualizerWindow::setControlsEnabled(bool isEnabled)
 {
-    for(int a=0; a<eq->getBandsCount(); a++)
+    for(int a=0; a<eq()->getBandsCount(); a++)
         ((QSlider*)bars[a]->children()[0])->setEnabled(isEnabled);
 }
 
@@ -309,12 +288,17 @@ void EqualizerWindow::checkForSave()
         onSaveClick();
 }
 
+Equalizer *EqualizerWindow::eq() const
+{
+    return PLAYER->getEqualizer();
+}
+
 void EqualizerWindow::onLevelChange(int level)
 {
     QSlider* slider = (QSlider*)(sender());
     QWidget* bar = (QWidget*)(slider->parent());
     QLabel* label = (QLabel*)(bar->children().at(2));
-    eq->setBandLevel(getBarIndex(bar), level);
+    eq()->setBandLevel(getBarIndex(bar), level);
     updateLevelLabel(label, level);
     isChanged = true;
 }
@@ -344,7 +328,7 @@ void EqualizerWindow::onNewClick()
                         );
             continue;
         }
-        if(eq->hasValidPreset(presetName))
+        if(eq()->hasValidPreset(presetName))
         {
             QMessageBox::critical(
                         this,
@@ -354,8 +338,8 @@ void EqualizerWindow::onNewClick()
             continue;
         }
 
-        eq->setDefaultLevels();
-        eq->saveToPreset(presetName);
+        eq()->setDefaultLevels();
+        eq()->saveToPreset(presetName);
         selectedPreset = presetName;
         updateLevels();
         updateSelectorLabel();
@@ -403,7 +387,7 @@ void EqualizerWindow::onRenameClick()
                         );
             continue;
         }
-        if(eq->hasValidPreset(presetName))
+        if(eq()->hasValidPreset(presetName))
         {
             QMessageBox::critical(
                         this,
@@ -413,7 +397,7 @@ void EqualizerWindow::onRenameClick()
             continue;
         }
 
-        eq->renamePreset(selectedPreset, presetName);
+        eq()->renamePreset(selectedPreset, presetName);
         selectedPreset = presetName;
         updateSelectorLabel();
         MAINWINDOW->showNotification(tr("The current preset has been renamed"));
@@ -432,7 +416,7 @@ void EqualizerWindow::onSaveClick()
                     );
         return;
     }
-    eq->saveToPreset(selectedPreset);
+    eq()->saveToPreset(selectedPreset);
     MAINWINDOW->showNotification(tr("The current preset has been updated"));
     isChanged = false;
 }
@@ -454,8 +438,8 @@ void EqualizerWindow::onDeleteClick()
                 tr("Are you sure you want to delete the current preset?")+"\n\""+selectedPreset+"\"",
                 QMessageBox::Ok, QMessageBox::Cancel))
     {
-        eq->deletePreset(selectedPreset);
-        eq->setDefaultLevels();
+        eq()->deletePreset(selectedPreset);
+        eq()->setDefaultLevels();
         selectedPreset = "";
         updateLevels();
         updateSelectorLabel();
@@ -469,7 +453,7 @@ void EqualizerWindow::onDeleteClick()
 void EqualizerWindow::onSelectorClick()
 {
     checkForSave();
-    QStringList presets = eq->getValidPresetsList();
+    QStringList presets = eq()->getValidPresetsList();
     int index = presets.indexOf(selectedPreset);
     index++;
     presets.prepend(presetOff);
