@@ -55,14 +55,13 @@ protected:
     void selectPreset(const QString& presetName);
     void setControlsEnabled(bool isEnabled);
     void checkForSave();
+    Equalizer* eq() const;
 
     QWidget* placeholder;
     QWidget** bars; // array of containers
     QPushButton* selector;
     Button *btnNew, *btnRename, *btnSave, *btnDelete;
 
-    Equalizer* eq;
-
     QString selectedPreset;
     QString presetOff;
     bool isChanged;
