@@ -21,7 +21,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 22 Apr 2012                                              }
+{ last modified: 21 May 2012                                              }
 {*************************************************************************/
 
 #include "mainwindow.h"
@@ -180,12 +180,11 @@ void MainWindow::initPlayer()
     connect(player, SIGNAL(onPlayEnd()), SLOT(onPlayEnd()));
     connect(player, SIGNAL(onStateChanged()), SLOT(onStateChanged()));
     connect(player, SIGNAL(onError()), SLOT(onError()));
-    connect(player, SIGNAL(onTitleChange(QString)), SLOT(onTitleChange(QString)));
+    connect(player, SIGNAL(onTitleChange()), SLOT(onTitleChange()));
     connect(player, SIGNAL(onHeadsetStatusChange(bool)), SLOT(onHeadsetStatusChange(bool)));
     connect(player, SIGNAL(onCallStart()), SLOT(onCallStart()));
     connect(player, SIGNAL(onCallEnd()), SLOT(onCallEnd()));
 
-    player->getEqualizer()->setPresetsIni(iniSettings);
     currentPreset = iniSettings->value("equalizer/currentPreset", "").toString();
     if(player->getEqualizer()->getEffect())
     {
@@ -242,7 +241,9 @@ void MainWindow::changeSkin(const QString &skinName)
     if(!skin->load(skinName))
         skin->load();
 
-    QString selWindows("MainWindow, OptionsWindow, SelectWindow, TypesWindow, EqualizerWindow, TimerWindow, AboutWindow");
+    QString selWindows(
+                "MainWindow, OptionsWindow, SelectWindow, TypesWindow, "
+                "EqualizerWindow, TimerWindow, AboutWindow, SearchWindow");
     QString selControls("QLabel, Button, QSlider, QButton");
 
     qApp->setStyleSheet(
@@ -368,8 +369,7 @@ bool MainWindow::cd(const QString &dir)
 
 void MainWindow::buildList()
 {
-    foreach(QObject* item, itemsContainer->children())
-        delete item;
+    itemsContainer->deleteAllChildren();
 
     QString path = fs->getCurrentDirectory();
     if(!path.endsWith("/"))
@@ -497,26 +497,33 @@ void MainWindow::updateListIcons(bool playMode)
     }
 }
 
-void MainWindow::play(const QString &filename)
+void MainWindow::play(const QString &filename, bool doCloseCurrent)
 {
     if(!OPTIONS.clearPlaylist)
     {
         int i = playlist.indexOf(filename);
         if(i >= 0)
         {
-            playIndex(i);
+            playIndex(i, doCloseCurrent);
             return;
         }
     }
+
+    QFileInfo info(filename);
+    QString currDir = fs->getCurrentDirectory();
+    QString targetDir = info.absoluteDir().canonicalPath();
+    fs->cd(targetDir);
     fs->getCurrentFilesFull(playlist);
+    fs->cd(currDir);
+
     straightPlaylist = playlist;
     if(OPTIONS.playMode == pmo_random)
         shuffle();
     errCount = 0;
-    playIndex(playlist.indexOf(filename));
+    playIndex(playlist.indexOf(filename), doCloseCurrent);
 }
 
-void MainWindow::playIndex(int index)
+void MainWindow::playIndex(int index, bool doCloseCurrent)
 {
     if((index >= playlist.size()) || (index < 0))
     {
@@ -527,9 +534,10 @@ void MainWindow::playIndex(int index)
 
     QString filename = playlist.at(index);
     currentFileIndex = index;
-    player->play(filename);
+    player->play(filename, doCloseCurrent);
 
     updateListIcons(true);
+    //cacheNext();
 }
 
 void MainWindow::stop()
@@ -539,9 +547,20 @@ void MainWindow::stop()
 
 void MainWindow::shuffle()
 {
+    QString curFilename = getCurrentFilename();
     int n = playlist.size();
     for(int a=0;a<n;a++)
         playlist.swap(a, rnd(n));
+    currentFileIndex = playlist.indexOf(curFilename);
+    cacheNext();
+}
+
+void MainWindow::straighten()
+{
+    QString curFilename = getCurrentFilename();
+    playlist = straightPlaylist;
+    currentFileIndex = playlist.indexOf(curFilename);
+    cacheNext();
 }
 
 void MainWindow::saveY()
@@ -600,6 +619,26 @@ void MainWindow::timerEvent(QTimerEvent *event)
     updateTimerActionLabel();
 }
 
+void MainWindow::keyPressEvent(QKeyEvent *event)
+{
+    int x = event->key();
+    if(
+        ((x < Qt::Key_A) || (x > Qt::Key_Z))
+            &&
+        ((x < Qt::Key_1) || (x > Qt::Key_9))
+            &&
+        (x != Qt::Key_Space)
+            &&
+        (x != Qt::Key_Enter)
+    )
+    {
+        return;
+    }
+
+    QString s = event->text().trimmed();
+    SearchWindow::showSearchWindow(s);
+}
+
 void MainWindow::onItemClick(QWidget *item)
 {
     KineticListItem* theItem = qobject_cast<KineticListItem*>(item);
@@ -628,6 +667,7 @@ void MainWindow::onContextMenuShow(QWidget *item, QContextMenuEvent *event)
     KineticListItem* i = qobject_cast<KineticListItem*>(item);
     if(i)
     {
+        contextMenuOwner = i;
         if(i->getBool())
             menuFolder->exec(event->globalPos());
         else
@@ -662,6 +702,7 @@ void MainWindow::onBtnPlayClick()
     switch(player->getState())
     {
         case pp_state_Idle:
+        case pp_state_Error:
             playIndex(currentFileIndex);
             break;
 
@@ -688,19 +729,15 @@ void MainWindow::onBtnParentClick()
 
 void MainWindow::onPlayEnd()
 {
-    if(OPTIONS.playMode == pmo_trackOnce)
-    {
-        if(currentFileItem)
-            currentFileItem->setIcon(skin->icoFileStop);
-        return;
-    }
+    if(currentFileItem)
+        currentFileItem->setIcon(skin->icoFileStop);
     if(!timerMinsLeft && OPTIONS.letTrackEnd)
     {
         stopByTimer();
         updateTimerActionLabel();
         return;
     }
-    if(!runNext())
+    if(!runNext(false))
         showStopNotification();
 }
 
@@ -755,6 +792,7 @@ void MainWindow::onStateChanged()
     switch(player->getState())
     {
         case pp_state_Idle:
+        case pp_state_Error:
             if(changeCurrentItem)
                 currentFileItem->setIcon(skin->icoFileStop);
             btnPlay->setIcon(skin->icoPlay);
@@ -777,12 +815,14 @@ void MainWindow::onStateChanged()
     }
 }
 
-void MainWindow::onTitleChange(const QString &title)
+void MainWindow::onTitleChange()
 {
+    cacheNext();
     if(OPTIONS.popupTrackTitles)
     {
         // all that nonsense below is just for fitting all the text in notification window
         // this circumvents a weird text cropping in such windows on Maemo
+        const QString& title = player->getTitle();
         QString cur, tot;
         cur.setNum(currentFileIndex+1);
         tot.setNum(playlist.size());
@@ -859,18 +899,12 @@ void MainWindow::onMenuOptionsClick()
         if(wasRandom)
         {
             if(!isRandom)
-            {
-                playlist = straightPlaylist;
-                updateCurrentFileIndex();
-            }
+                straighten();
         }
         else
         {
             if(isRandom)
-            {
                 shuffle();
-                updateCurrentFileIndex();
-            }
         }
     }
     updateScreenOrientation();
@@ -923,8 +957,7 @@ void MainWindow::onMenuFolderAllClick()
     stop();
     playlist.clear();
     QString curdir = fs->getCurrentDirectory();
-    KineticListItem* item = (KineticListItem*)(itemsContainer->getActiveWidget());
-    addFromDir(item->getString());
+    addFromDir(contextMenuOwner->getString());
     showNotification(tr("found %n file(s)", "", playlist.size()));
     straightPlaylist = playlist;
 
@@ -938,10 +971,9 @@ void MainWindow::onMenuFolderAllAddClick()
 {
     showNotification(tr("searching..."));
     QString curdir = fs->getCurrentDirectory();
-    KineticListItem* item = (KineticListItem*)(itemsContainer->getActiveWidget());
     playlist = straightPlaylist;
     int n = playlist.size();
-    addFromDir(item->getString());
+    addFromDir(contextMenuOwner->getString());
     playlist.removeDuplicates();
     n = playlist.size() - n;
     QString s;
@@ -954,28 +986,30 @@ void MainWindow::onMenuFolderAllAddClick()
     showNotification(s);
     straightPlaylist = playlist;
 
+    QString curFilename = getCurrentFilename();
     if(OPTIONS.playMode == pmo_random)
         shuffle();
-    updateCurrentFileIndex();
+    currentFileIndex = playlist.indexOf(curFilename);
     fs->cd(curdir);
 }
 
 void MainWindow::onMenuFileSetNextClick()
 {
     queue.clear();
-    KineticListItem* item = (KineticListItem*)(itemsContainer->getActiveWidget());
-    queue.append(item->getString());
+    queue.append(contextMenuOwner->getString());
     showNotification(tr("this file will be played next"));
+    cacheNext();
 }
 
 void MainWindow::onMenuFileAddQueueClick()
 {
-    KineticListItem* item = (KineticListItem*)(itemsContainer->getActiveWidget());
-    queue.append(item->getString());
+    queue.append(contextMenuOwner->getString());
     QString s;
     s.setNum(queue.size());
     s.prepend(tr("files queued")+": ");
     showNotification(tr("this file has been added to queue")+"\n"+s);
+    if(queue.size() == 1)
+        cacheNext();
 }
 
 void MainWindow::onHeadsetStatusChange(bool isConnected)
@@ -1087,6 +1121,11 @@ void MainWindow::jumpToCurrentItem()
     }
 }
 
+void MainWindow::cacheNext()
+{
+    QTimer::singleShot(1000, this, SLOT(doCacheNext()));
+}
+
 void MainWindow::addFromDir(const QString &dirname)
 {
     if(!fs->cd(dirname))
@@ -1149,14 +1188,14 @@ void MainWindow::updateEqualizerActionLabel()
     actionEqualizer->setText(tr("Equalizer")+": "+presetName);
 }
 
-bool MainWindow::runNext()
+bool MainWindow::runNext(bool doCloseCurrent)
 {
     isMovingBackwards = false;
     cancelResumeCheck();
 
     if(!queue.isEmpty())
     {
-        play(queue.takeFirst());
+        play(queue.takeFirst(), doCloseCurrent);
         return true;
     }
 
@@ -1188,7 +1227,7 @@ bool MainWindow::runNext()
             return false;
     }
 
-    playIndex(i);
+    playIndex(i, doCloseCurrent);
     return true;
 }
 
@@ -1261,6 +1300,37 @@ void MainWindow::endTimer()
     killTheTimer();
 }
 
+void MainWindow::doCacheNext()
+{
+    if(currentFileIndex < 0)
+        return;
+    if(playlist.isEmpty())
+        return;
+    if(queue.isEmpty() && (OPTIONS.playMode == pmo_trackOnce))
+        return;
+
+    int i;
+    if(!queue.isEmpty())
+    {
+        i = playlist.indexOf(queue.at(0));
+        if(i < 0)
+            return;
+    }
+    else
+    {
+        i = currentFileIndex + 1;
+        if(i >= playlist.size())
+        {
+            if((OPTIONS.playMode == pmo_random) || (OPTIONS.playMode == pmo_playlistLoop))
+                i = 0;
+            else
+                return;
+        }
+    }
+    QString cacheFilename = playlist.at(i);
+    player->cache(cacheFilename);
+}
+
 void MainWindow::showStopTimerNotification()
 {
     showNotification(tr("Timer has been stopped"));
