@@ -21,7 +21,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 25 Mar 2012                                              }
+{ last modified: 26 May 2012                                              }
 {*************************************************************************/
 
 #include "phononplayer.h"
@@ -30,19 +30,18 @@
 PhononPlayer::PhononPlayer(QObject *parent) :
     QObject(parent)
 {
-    knownPlaylistExtensions << "m3u" << "m3u8" << "pls";
     updateKnownMime();
     updateSupportedMime();
     findUnzip();
-    dev = NULL;
-    state = pp_state_Idle;
-    sourceType = pp_stype_None;
 
     equalizer = new Equalizer(this);
+    equalizer->setPresetsIni(MAINWINDOW->getIniSettings());
     equalizerEnabled = false;
+
     lastPos = -1;
 
-    createMediaObject();
+    sound = new PhononSound(this); // just to prevent errors when getting properties
+    cacheSound = NULL;
 
     tmpFilenameBase = QDesktopServices::storageLocation(QDesktopServices::TempLocation)+"/___"+APP_NAME+"_temp.";
 
@@ -55,16 +54,6 @@ PhononPlayer::PhononPlayer(QObject *parent) :
                     this,
                     SLOT(onHeadsetCondition())
                     );
-/*
-    QDBusConnection::systemBus().connect(
-                    "org.freedesktop.Hal",
-                    "/org/freedesktop/Hal/Manager",
-                    "org.freedesktop.Hal.Manager",
-                    "DeviceAdded",
-                    this,
-                    SLOT(onDeviceAdd())
-                    );
-*/
 
     QDBusConnection::systemBus().connect(
                     MCE_SERVICE,
@@ -90,48 +79,66 @@ PhononPlayer::PhononPlayer(QObject *parent) :
 
 PhononPlayer::~PhononPlayer()
 {
-    close();
+    stop();
     delete equalizer;
-    delete mediaObject;
-    delete audioOutput;
+    delete sound;
+    delete cacheSound;
 }
 
-bool PhononPlayer::play(const QString &filename)
+bool PhononPlayer::play(const QString &filename, bool doCloseCurrent)
 {
-    if(filename.indexOf("://") >= 0)
+    killLater(sound, doCloseCurrent);
+    if(!loadFromCache(filename))
     {
-        if(!filename.startsWith("http://"))
-            return false;
-        // TODO: network stream
-        //sourceType = pp_stype_None;
-        return false;
+        sound = new PhononSound(this, equalizerEnabled);
+        sound->open(filename);
     }
     else
     {
-        close();
-        mediaObject->setCurrentSource(Phonon::MediaSource(filename));
-
-        currentFilename = filename;
-
-        sourceType = pp_stype_File;
-        mediaObject->play();
+        if(sound->getState() == pp_state_Error)
+        {
+            QTimer::singleShot(0, this, SIGNAL(onError()));
+            return true;
+        }
     }
+
+    connect(sound, SIGNAL(onPlayEnd()), this, SIGNAL(onPlayEnd()));
+    connect(sound, SIGNAL(onError()), this, SIGNAL(onError()));
+    connect(sound, SIGNAL(onStateChanged()), this, SIGNAL(onStateChanged()));
+    connect(sound, SIGNAL(onTitleChange()), this, SIGNAL(onTitleChange()));
+    sound->play();
+    if(!sound->getTitle().isEmpty())
+        emit onTitleChange();
+
     return true;
 }
 
+void PhononPlayer::cache(const QString &filename)
+{
+    // there's no way to make a decent pre-caching with equalizer enabled
+    // due to glitches in playback when two equalizers are active at one time
+    if(equalizerEnabled)
+        return;
+    if(cacheExists(filename))
+        return;
+    killLater(cacheSound);
+    cacheSound = new PhononSound(this);
+    cacheSound->open(filename);
+}
+
 void PhononPlayer::pause()
 {
-    mediaObject->pause();
+    sound->pause();
 }
 
 void PhononPlayer::unpause()
 {
-    mediaObject->play();
+    sound->play();
 }
 
 void PhononPlayer::stop()
 {
-    close();
+    sound->close();
 }
 
 bool PhononPlayer::isHeadsetConnected()
@@ -149,50 +156,15 @@ bool PhononPlayer::isHeadsetConnected()
 
 bool PhononPlayer::setEqualizerEnabled(bool doEnable)
 {
-    if(!equalizer->getEffect())
+    if(!getEqualizer()->getEffect())
         return false;
-    if(doEnable == equalizerEnabled)
-        return true;
-    // TODO:
-    // when applying/deleting effect, the mediaSource is reset,
-    // so there is need to restore its previous position
-    /*
-    if(state != pp_state_Idle)
-        lastPos = mediaObject->currentTime();
-    else
-        lastPos = -1;
-    */
-    mediaPath.removeEffect(equalizer->getEffect()); // always remove the effect, just in case
-    if(doEnable)
-        mediaPath.insertEffect(equalizer->getEffect());
+    sound->setEqualizerEnabled(doEnable);
+    killLater(cacheSound, true);
+    cacheSound = NULL;
     equalizerEnabled = doEnable;
     return true;
 }
 
-void PhononPlayer::close(bool emitStop)
-{
-    state = pp_state_Idle;
-    mediaObject->stop();
-    mediaObject->clearQueue();
-    if(dev)
-    {
-        dev->close();
-        dev = NULL;
-    }
-    sourceType = pp_stype_None;
-
-    // the whole thing below is
-    // a workaround for a one strange gstreamer bug, which
-    // is not allowing to play *.ogg after playing *.flac
-    if(emitStop)
-        stateChanged(Phonon::StoppedState, mediaObject->state()); // in case the stop event had not been fired yet
-    // because we're calling the current function from a mediaObject's slot;
-    // delete audioOutput as well, because, it will not be possible to make a path otherwise;
-    killLater(mediaObject); // also, sometimes, there are errors popping out of nowhere, when using deleteLater,
-    killLater(audioOutput); // so we delete objects after 10 seconds (in killLater function)
-    createMediaObject(); // create mediaObject anew
-}
-
 void PhononPlayer::updateSupportedMime()
 {
     supportedMime = Phonon::BackendCapabilities::availableMimeTypes();
@@ -272,72 +244,6 @@ void PhononPlayer::findUnzip()
     unzipPath = FSMan::which("unzip");
 }
 
-void PhononPlayer::stateChanged(Phonon::State newstate, Phonon::State oldstate)
-{
-    switch(newstate)
-    {
-        case Phonon::ErrorState:
-            qDebug() << currentFilename << mediaObject->errorString();
-            if(mediaObject->errorType() == Phonon::FatalError)
-            {
-                if(sourceType == pp_stype_File)
-                {
-                    if(unzip(currentFilename))
-                    {
-                        close(false);
-                        mediaObject->setCurrentSource(Phonon::MediaSource(tmpFilename));
-                        //mediaObject->setCurrentSource(Phonon::MediaSource(dev));
-                        //sourceType = pp_stype_Memory;
-                        mediaObject->play();
-                        return;
-                    }
-                }
-                close();
-                onError();
-            }
-            break;
-
-        case Phonon::StoppedState:
-            if(
-                (oldstate != Phonon::PlayingState) &&
-                (oldstate != Phonon::PausedState)
-            )
-            {
-                break;
-            }
-            state = pp_state_Idle;
-            emit onStateChanged();
-            break;
-
-        case Phonon::PlayingState:
-            state = pp_state_Playing;
-            emit onStateChanged();
-            break;
-
-        case Phonon::PausedState:
-            if(
-                (oldstate != Phonon::PlayingState) ||
-                (state != pp_state_Playing)
-            )
-            {
-                break;
-            }
-            state = pp_state_Paused;
-            emit onStateChanged();
-            break;
-
-        default:
-            break;
-    }
-}
-
-void PhononPlayer::onCustomPlayEnd()
-{
-    state = pp_state_Idle;
-    emit onStateChanged();
-    emit onPlayEnd();
-}
-
 void PhononPlayer::onHeadsetCondition()
 {
     if(!isHeadsetConnected())
@@ -346,11 +252,6 @@ void PhononPlayer::onHeadsetCondition()
         QTimer::singleShot(3000, this, SLOT(onCheckHeadset())); // safe interval
 }
 
-void PhononPlayer::onDeviceAdd()
-{
-    // ^^^
-}
-
 void PhononPlayer::onCheckHeadset()
 {
     if(isHeadsetConnected())
@@ -379,47 +280,7 @@ void PhononPlayer::onKillObsoleteObject()
         obsoleteObjects.takeFirst()->deleteLater();
 }
 
-void PhononPlayer::onMetaDataChanged()
-{
-    QString title;
-    QStringList titles = mediaObject->metaData(Phonon::TitleMetaData);
-    if(!titles.isEmpty())
-        title = titles.at(0);
-    if(title.isEmpty())
-    {
-        QFileInfo info(currentFilename);
-        title = info.completeBaseName();
-    }
-    QString artist;
-    QStringList artists = mediaObject->metaData(Phonon::ArtistMetaData);
-    if(!artists.isEmpty())
-        artist = artists.at(0);
-    QString fullTitle;
-    if(artist.isEmpty())
-        fullTitle = title;
-    else
-        fullTitle = artist+" - "+title;
-    if(lastSentFullTitle == fullTitle)
-        return; // to prevent multiple events with a same title
-    emit onTitleChange(fullTitle);
-    lastSentFullTitle = fullTitle;
-}
-
-void PhononPlayer::createMediaObject()
-{
-    mediaPath.removeEffect(equalizer->getEffect()); // always remove the effect, just in case
-    mediaObject = new Phonon::MediaObject(this);
-    audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
-    mediaPath = Phonon::createPath(mediaObject, audioOutput);
-    if(equalizerEnabled)
-        mediaPath.insertEffect(equalizer->getEffect());
-    connect(mediaObject, SIGNAL(finished()), SLOT(onCustomPlayEnd()));
-    connect(mediaObject, SIGNAL(stateChanged(Phonon::State,Phonon::State)), SLOT(stateChanged(Phonon::State,Phonon::State)));
-    connect(mediaObject, SIGNAL(metaDataChanged()), SLOT(onMetaDataChanged()));
-    lastSentFullTitle = "";
-}
-
-bool PhononPlayer::unzip(const QString &filename)
+bool PhononPlayer::unzip(const QString &filename, QString& tmpFilename)
 {
     if(unzipPath.isEmpty())
         return false;
@@ -443,16 +304,6 @@ bool PhononPlayer::unzip(const QString &filename)
         return false;
 
     QByteArray mem;
-
-    /*
-    QProcess proc;
-    proc.setStandardInputFile(filename);
-    proc.start(unzipPath);
-    if(!proc.waitForFinished())
-        return false;
-
-    mem = proc.readAllStandardOutput();
-    */
     FSMan::getRawProcessOutput(mem, unzipPath, (QStringList() << "-p" << filename));
 
     int n = mem.size();
@@ -460,48 +311,50 @@ bool PhononPlayer::unzip(const QString &filename)
         return false; // no output was returned
 
     // we must create an actual file to circumvent most
-    // gstreamer problems regarding "magick" file identification
+    // phonon problems regarding "magick" file identification
     QFileInfo info(filename);
-    tmpFilename = tmpFilenameBase + info.suffix();
+    tmpFilename = tmpFilenameBase + randomHash() + "." + info.suffix();
     QFile tmp(tmpFilename);
     if(!tmp.open(QIODevice::WriteOnly))
         return false;
-    if(tmp.write(mem) != n)
+    int w = tmp.write(mem);
+    if(w != n)
     {
         tmp.close();
         return false;
     }
     tmp.close();
 
-/*
-    close();
-
-    if(!devMem.open(QIODevice::ReadWrite))
-        return false;
-    if(!devMem.seek(0))
-    {
-        devMem.close();
-        return false;
-    }
-    if(devMem.write(mem) != n)
-    {
-        devMem.close();
-        return false;
-    }
-    if(!devMem.seek(0))
-    {
-        devMem.close();
-        return false;
-    }
-
-    dev = &devMem;
-*/
     return true;
 }
 
-void PhononPlayer::killLater(QObject *obsoleteObject)
+void PhononPlayer::killLater(PhononSound *soundObject, bool doClose)
 {
-    obsoleteObject->disconnect();
-    obsoleteObjects.append(obsoleteObject);
-    QTimer::singleShot(10000, this, SLOT(onKillObsoleteObject())); // 10 seconds more to live
+    if(!soundObject)
+        return;
+    soundObject->prepareToDie(doClose);
+    obsoleteObjects.append(soundObject);
+    QTimer::singleShot(15000, this, SLOT(onKillObsoleteObject())); // 15 seconds more to live
+}
+
+bool PhononPlayer::loadFromCache(const QString &filename)
+{
+    // there's no way to make a decent pre-caching with equalizer enabled
+    // due to glitches in playback when two equalizers are active at one time
+    if(equalizerEnabled)
+        return false;
+    if(!cacheExists(filename))
+        return false;
+    sound = cacheSound;
+    cacheSound = NULL;
+    return true;
+}
+
+bool PhononPlayer::cacheExists(const QString &filename)
+{
+    if(!cacheSound)
+        return false;
+    if(cacheSound->getCurrentFilename() != filename)
+        return false;
+    return true;
 }
