@@ -21,7 +21,7 @@
 { You may read GNU General Public License at:                             }
 {   http://www.gnu.org/copyleft/gpl.html                                  }
 {                                                                         }
-{ last modified: 31 Mar 2012                                              }
+{ last modified: 26 May 2012                                              }
 {*************************************************************************/
 
 #ifndef PHONONPLAYER_H
@@ -29,19 +29,7 @@
 
 #include "includes.h"
 #include "equalizer.h"
-
-typedef enum {
-    pp_state_Idle,
-    pp_state_Playing,
-    pp_state_Paused
-} PhononPlayerState;
-
-typedef enum {
-    pp_stype_None,
-    pp_stype_File,
-    pp_stype_Memory,
-    pp_stype_Remote
-} PhononPlayerSourceType;
+#include "phononsound.h"
 
 class PhononPlayer : public QObject
 {
@@ -52,13 +40,14 @@ public:
     inline const QStringList& getKnownExtensions() const {return knownExtensions;}
     inline const QStringList& getSupportedMime() const {return supportedMime;}
     inline const QStringList& getSupportedExtensions() const {return supportedExtensions;}
-    inline const QStringList& getKnownPlaylistExtensions() const {return knownPlaylistExtensions;}
     inline const QString& getUnzipPath() const {return unzipPath;}
-    inline const QString& getCurrentFilename() const {return currentFilename;}
-    inline Equalizer* getEqualizer(){return equalizer;}
-    inline PhononPlayerState getState() const {return state;}
-    inline PhononPlayerSourceType getSourceType() const {return sourceType;}
-    bool play(const QString& filename);
+    inline const QString& getCurrentFilename() const {return sound->getCurrentFilename();}
+    inline Equalizer* getEqualizer() const {return equalizer;}
+    inline PhononSoundState getState() const {return sound->getState();}
+    inline PhononSoundSourceType getSourceType() const {return sound->getSourceType();}
+    inline const QString& getTitle() const {return sound->getTitle();}
+    bool play(const QString& filename, bool doCloseCurrent = true);
+    void cache(const QString& filename);
     void pause();
     void unpause();
     void stop();
@@ -66,10 +55,10 @@ public:
     inline bool isEqualizerEnabled() const {return equalizerEnabled;}
     bool setEqualizerEnabled(bool doEnable);
     inline bool isCalling() const {return calling;}
+    bool unzip(const QString& filename, QString& tmpFilename);
 
 public slots:
     void updateSupportedMime();
-    void close(bool emitStop = true);
 
 signals:
     void onPlayEnd();
@@ -78,22 +67,19 @@ signals:
     void onHeadsetStatusChange(bool isConnected);
     void onCallStart();
     void onCallEnd();
-    void onTitleChange(const QString& title);
+    void onTitleChange();
 
 protected:
-    void createMediaObject();
     void updateKnownMime();
     void findUnzip();
     void updateNameFilters();
-    bool unzip(const QString& filename);
-    void killLater(QObject* obsoleteObject);
+    void killLater(PhononSound *soundObject, bool doClose = true);
+    bool loadFromCache(const QString &filename);
+    bool cacheExists(const QString &filename);
 
-    QFile devFile;
-    QBuffer devMem;
-    QIODevice* dev;
-    PhononPlayerState state;
-    PhononPlayerSourceType sourceType;
+    PhononSound *sound, *cacheSound;
     Equalizer* equalizer;
+
     bool equalizerEnabled;
     qint64 lastPos;
     bool calling;
@@ -102,28 +88,15 @@ protected:
     QStringList knownExtensions;
     QStringList supportedMime;
     QStringList supportedExtensions;
-    QStringList knownPlaylistExtensions;
     QString unzipPath;
     QString tmpFilenameBase;
-    QString tmpFilename;
     QObjectList obsoleteObjects;
 
-    Phonon::MediaObject *mediaObject;
-    Phonon::AudioOutput *audioOutput;
-    Phonon::Path mediaPath;
-    QString currentFilename;
-    QString lastSentFullTitle;
-
 protected slots:
-    void stateChanged(Phonon::State newstate, Phonon::State oldstate);
-    void onCustomPlayEnd();
     void onHeadsetCondition();
-    void onDeviceAdd();
     void onCheckHeadset();
     void onCallStateChange(QString callState);
     void onKillObsoleteObject();
-    void onMetaDataChanged();
-
 };
 
 #endif // PHONONPLAYER_H
