@@ -125,27 +125,32 @@
         <translation>Чтобы иметь возможность воспроизводить сжатую треккерную музыку (*.xmz, *.s3z, ...), вы должны установить &quot;unzip&quot;. Этот пакет может быть найден в репозитории &quot;extras-testing&quot;.</translation>
     </message>
     <message>
-        <location filename="../src/aboutwindow.cpp" line="163"/>
+        <location filename="../src/aboutwindow.cpp" line="118"/>
+        <source>In search window you can press Enter key instead of the &quot;search&quot; button. You can also stop a current search by pressing Enter key.</source>
+        <translation>В окне поиска вы можете нажимать клавишу Enter вместо кнопки &quot;поиск&quot;. Вы также можете остановить начатый поиск нажатием клавиши Enter.</translation>
+    </message>
+    <message>
+        <location filename="../src/aboutwindow.cpp" line="166"/>
         <source>About</source>
         <translation>О программе</translation>
     </message>
     <message>
-        <location filename="../src/aboutwindow.cpp" line="173"/>
+        <location filename="../src/aboutwindow.cpp" line="176"/>
         <source>App</source>
         <translation>Приложение</translation>
     </message>
     <message>
-        <location filename="../src/aboutwindow.cpp" line="174"/>
+        <location filename="../src/aboutwindow.cpp" line="177"/>
         <source>System</source>
         <translation>Система</translation>
     </message>
     <message>
-        <location filename="../src/aboutwindow.cpp" line="175"/>
+        <location filename="../src/aboutwindow.cpp" line="178"/>
         <source>Skin</source>
         <translation>Скин</translation>
     </message>
     <message>
-        <location filename="../src/aboutwindow.cpp" line="176"/>
+        <location filename="../src/aboutwindow.cpp" line="179"/>
         <source>Tips</source>
         <translation>Подсказки</translation>
     </message>
@@ -611,12 +616,12 @@
         <translation>ничего не найдено</translation>
     </message>
     <message>
-        <location filename="../src/searchwindow.cpp" line="144"/>
+        <location filename="../src/searchwindow.cpp" line="159"/>
         <source>search</source>
         <translation>поиск</translation>
     </message>
     <message>
-        <location filename="../src/searchwindow.cpp" line="176"/>
+        <location filename="../src/searchwindow.cpp" line="201"/>
         <source>stop</source>
         <translation>стоп</translation>
     </message>
